{# Support for Sphinx 1.3+ page_source_suffix, but don't break old builds. #}

<div role="navigation" aria-label="breadcrumbs navigation">

  <ul class="wy-breadcrumbs">
    {% block breadcrumbs %}
      <li><a href="{{ pathto(master_doc) }}">{{ _('Docs') }}</a> &raquo;</li>
        {% for doc in parents %}
          <li><a href="{{ doc.link|e }}">{{ doc.title }}</a> &raquo;</li>
        {% endfor %}
      <li>{{ title }}</li>
    {% endblock %}
    {% block breadcrumbs_aside %}
      <li class="wy-breadcrumbs-aside">
      </li>
    {% endblock %}
  </ul>

  <script>
    // position the github ribbon relative to wy-nav-content element:
    document.querySelector('.wy-nav-content').setAttribute(
      "style", "position: absolute");
    // fix color bug caused by positioning hack (...?):
    document.querySelector('.wy-nav-content-wrap').setAttribute(
      "style", "background: #fcfcfc");
  </script>

  <a href="https://github.com/{{ github_user }}/{{ github_repo }}"><img style="position: absolute; top: 0; right: 0; border: 0;" src="https://s3.amazonaws.com/github/ribbons/forkme_right_red_aa0000.png" alt="Fork me on GitHub"></a>

  {% if (theme_prev_next_buttons_location == 'top' or theme_prev_next_buttons_location == 'both') and (next or prev) %}
  <div class="rst-breadcrumbs-buttons" role="navigation" aria-label="breadcrumb navigation">
      {% if next %}
        <a href="{{ next.link|e }}" class="btn btn-neutral float-right" title="{{ next.title|striptags|e }}" accesskey="n">Next <span class="fa fa-arrow-circle-right"></span></a>
      {% endif %}
      {% if prev %}
        <a href="{{ prev.link|e }}" class="btn btn-neutral" title="{{ prev.title|striptags|e }}" accesskey="p"><span class="fa fa-arrow-circle-left"></span> Previous</a>
      {% endif %}
  </div>
  {% endif %}
  <hr/>
</div>
